/**************************************************************************
 *   Copyright (C) 2009 by Peter Semiletov                                 *
 *   peter.semiletov@gmail.com                                             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "mainwindow.h"
#include "utils.h"


MainWindow::MainWindow (QWidget *parent): QMainWindow (parent)
{
  settings = new QSettings ("neformal", "neformal");

  enc_window = NULL;
  
  qtTranslator.load (QString ("qt_%1").arg (QLocale::system().name()),
                     QLibraryInfo::location (QLibraryInfo::TranslationsPath));
  qApp->installTranslator (&qtTranslator);

  myappTranslator.load (":/translations/neformal_" + QLocale::system().name());
  qApp->installTranslator (&myappTranslator);
   
  main_tab_widget = new QTabWidget;
  main_tab_widget->setTabPosition (QTabWidget::East);
  setCentralWidget (main_tab_widget);
  
  create_paths();
  create_actions();
  setup_mime_table();
  create_ctl_player();
  create_fman();
  create_menus();
  update_themes_menu();
  readSettings();
  
  img_viewer = new CImgViewer;
  
  QWidget *tab_play = new QWidget();
  QVBoxLayout *lv_tab_play = new QVBoxLayout();
  tab_play->setLayout (lv_tab_play);
  
  ple_memo = new QPlainTextEdit;
  log = new CLogMemo (ple_memo);

  
  spl_main = new QSplitter (Qt::Vertical, this);
  spl_main->addWidget (wd_fman);
  spl_main->addWidget (ple_memo);

  lv_tab_play->addWidget (ctl_player);
  lv_tab_play->addWidget (spl_main);
  
  main_tab_widget->addTab (tab_play, tr ("play"));

  create_manual();
  
  restoreState (settings->value ("state", QByteArray()).toByteArray());
  spl_main->restoreState (settings->value ("spl_main").toByteArray());

  text_file_browser = new QPlainTextEdit;
  text_file_browser->setReadOnly (true);
  
  qApp->setWindowIcon (QIcon (":/icons/app.svg"));
 
  if (QSystemTrayIcon::isSystemTrayAvailable())
     {
      ico_tray = new QSystemTrayIcon (QIcon (":/icons/app.svg"), this);
      ico_tray->show(); 
      connect (ico_tray, SIGNAL(activated ( QSystemTrayIcon::ActivationReason)), 
              this, SLOT(tray_activated ( QSystemTrayIcon::ActivationReason)));
     }   
  
  log->log (tr ("<b>Neformal %1 @ http://neformal.sourceforge.net</b><br>by Peter Semiletov (peter.semiletov@gmail.com)<br>read the Manual under the <i>learn</i> tab!").arg (QString (VERSION_NUMBER)));
  setWindowTitle (tr ("neformal"));
  
  QString vn = settings->value ("VERSION_NUMBER", "").toString();
  if (vn.isEmpty() || vn != QString (VERSION_NUMBER))
      show_news();
}


void MainWindow::tray_activated (QSystemTrayIcon::ActivationReason reason)
{
  if (isVisible())
     hide();
  else
      showNormal();
}


void MainWindow::changeEvent (QEvent *event)
{
 // if (event->type() == QEvent::WindowStateChange)
   //  if (isMinimized())
     //    hide();
}


MainWindow::~MainWindow()
{
  writeSettings();
  delete text_file_browser;
  delete fman;
  delete settings;
  delete log;
}


void MainWindow::fman_drives_changed (const QString & path)
{
  if (! ui_update)
     fman->nav (path);
}


void MainWindow::fman_naventry_confirm()
{
  fman->nav (ed_fman_path->text());
}


void MainWindow::fman_refresh_dir()
{
   fman->refresh();
}


void MainWindow::create_fman()
{
  wd_fman = new QWidget;

  QVBoxLayout *lav_main = new QVBoxLayout;
  QHBoxLayout *lah_topbar = new QHBoxLayout;
  
  ed_fman_path = new QLineEdit;
  connect (ed_fman_path, SIGNAL(returnPressed()), this, SLOT(fman_naventry_confirm()));

#ifdef Q_WS_WIN

  cb_fman_drives = new QComboBox;
  lah_topbar->addWidget (cb_fman_drives);

  QFileInfoList l_drives = QDir::drives();
  foreach (QFileInfo fi, l_drives)
           cb_fman_drives->addItem (fi.path());

  connect (cb_fman_drives, SIGNAL(currentIndexChanged ( const QString & )),
          this, SLOT(fman_drives_changed(const QString & )));

  
#endif

  lah_topbar->addWidget (ed_fman_path);
  lav_main->addLayout (lah_topbar);
  
  
  fman = new CFMan;

  connect (fman, SIGNAL(file_activated (const QString &)), this, SLOT(fman_file_activated (const QString &)));
  connect (fman, SIGNAL(dir_changed  (const QString &)), this, SLOT(fman_dir_changed  (const QString &)));


  w_right = new QWidget;
  
  QVBoxLayout *lw_right = new QVBoxLayout;
  w_right->setLayout (lw_right);  
  

  QLabel *l_bookmarks = new QLabel (tr ("<b>Bookmarks</b>"));
  lw_right->addWidget (l_bookmarks);

  QHBoxLayout *lah_places_bar = new QHBoxLayout;
  QPushButton *bt_add_bmk = new QPushButton ("+");
  QPushButton *bt_del_bmk = new QPushButton ("-");
  lah_places_bar->addWidget (bt_add_bmk);
  lah_places_bar->addWidget (bt_del_bmk);

  connect (bt_add_bmk, SIGNAL(clicked()), this, SLOT(fman_add_bmk()));
  connect (bt_del_bmk, SIGNAL(clicked()), this, SLOT(fman_del_bmk()));

  lv_places = new QListWidget;
  update_places_bookmarks();
  connect (lv_places, SIGNAL(itemActivated (QListWidgetItem *)), this, SLOT(fman_places_itemActivated (QListWidgetItem *)));

  lw_right->addLayout (lah_places_bar);
  lw_right->addWidget (lv_places);

  
  fman->tree_view->setSizePolicy (QSizePolicy::MinimumExpanding, QSizePolicy::Maximum);
  
  spl_fman = new QSplitter (this);
  spl_fman->setChildrenCollapsible (true);
    
  spl_fman->addWidget (fman->tree_view);
  spl_fman->addWidget (w_right);
  
  lav_main->addWidget (spl_fman);
  
  
  wd_fman->setLayout (lav_main);

  spl_fman->restoreState (settings->value ("spl_fman").toByteArray());
  
  QString hpath;

#ifdef Q_WS_WIN

  hpath = "c:\\";
  
#else  
  
  hpath = QDir::homePath();
  
#endif

  QString last_path = settings->value ("last_dir", hpath).toString();
  
  if (QFileInfo (last_path).exists())
     fman->nav (last_path);
  else
      fman->home();
}
  

void MainWindow::fman_file_activated (const QString &full_path)
{
   if (is_image (full_path))
     {
      img_viewer->window_full.show();
      img_viewer->set_image_full (full_path);
     }
  else 
  if (is_file_supported_by_mime (full_path))
     {
      mediaObject->stop();
      mediaObject->clearQueue();
      mediaObject->setCurrentSource (Phonon::MediaSource (full_path));
      mediaObject->play();
     }
  else
      if (is_text (full_path))
         show_text_file_enc (full_path);
}


void MainWindow::readSettings()
{
  tags_format = settings->value ("tags_format", "%ARTIST - %TITLE").toString();
  charset = settings->value ("charset", "UTF-8").toString();
  
  QPoint pos = settings->value ("pos", QPoint (1, 200)).toPoint();
  QSize size = settings->value ("size", QSize (600, 420)).toSize();
  resize (size);
  move (pos);

  QApplication::setStyle (QStyleFactory::create (settings->value ("ui_theme", "Cleanlooks").toString()));
}


void MainWindow::writeSettings()
{
  settings->setValue ("pos", pos());
  settings->setValue ("size", size());
  settings->setValue ("spl_fman", spl_fman->saveState());
  settings->setValue ("VERSION_NUMBER", QString (VERSION_NUMBER));
  settings->setValue ("state", saveState());
  settings->setValue ("tags_format", tags_format);
  settings->setValue ("last_dir", fman->dir.absolutePath());
  settings->setValue ("charset", charset);
  settings->setValue ("spl_main", spl_main->saveState());
}


void MainWindow::create_paths()
{

  QDir dr;
  dir_config = dr.homePath();

#ifdef Q_WS_WIN

  dir_config.append ("/neformal");

#else

  dir_config.append ("/.config/neformal");

#endif

  dr.setPath (dir_config);
  if (! dr.exists())
     dr.mkpath (dir_config);
  
  fname_places_bookmarks.append (dir_config).append ("/places_bookmarks");
}


void MainWindow::create_ctl_player()
{
  audioOutput = new Phonon::AudioOutput (Phonon::MusicCategory, this);
  mediaObject = new Phonon::MediaObject (this);

  mediaObject->setTickInterval (1000);

  connect (mediaObject, SIGNAL(tick(qint64)), this, SLOT(tick(qint64)));
    
  connect (mediaObject, SIGNAL(currentSourceChanged(const Phonon::MediaSource &)),
           this, SLOT(sourceChanged(const Phonon::MediaSource &)));
    
  connect(mediaObject, SIGNAL(aboutToFinish()), this, SLOT(aboutToFinish()));
  connect(mediaObject, SIGNAL(metaDataChanged()), this, SLOT(metaDataChanged()));
  connect(mediaObject, SIGNAL(totalTimeChanged(qint64 )), this, SLOT(totalTimeChanged(qint64 )));
  
  Phonon::createPath (mediaObject, audioOutput);

 
 
  ctl_player = new QWidget (this);
  
  l_time = new QLabel ("00:00");
  l_total_time = new QLabel ("00:00");
  
  seek_slider = new Phonon::SeekSlider;
  
  QHBoxLayout *seekerLayout = new QHBoxLayout;
  seekerLayout->addWidget (seek_slider);
  seekerLayout->addWidget (l_time);
  seekerLayout->addWidget (l_total_time);

  
  QHBoxLayout *playbackLayout = new QHBoxLayout;
  lv_player_layout = new QVBoxLayout;
  lv_player_layout->addLayout (playbackLayout);
  lv_player_layout->addLayout (seekerLayout);
    
  ctl_player->setLayout (lv_player_layout);
    
  seek_slider->setMediaObject (mediaObject);

  
  tb_player = addToolBar (tr ("Player"));
  tb_player->setObjectName ("tb_player");
  tb_player->addAction (act_prev);
  tb_player->addAction (act_stop);
  tb_player->addAction (act_play);
  tb_player->addAction (act_next);
    
  tb_volume = addToolBar (tr ("Volume"));
  tb_volume->setObjectName ("tb_volume");
  
  slider_volume = new Phonon::VolumeSlider (this);
  slider_volume->setAudioOutput (audioOutput);
  slider_volume->setSizePolicy(QSizePolicy::Maximum, QSizePolicy::Maximum);
    
  tb_volume->addWidget (slider_volume);
 }


void MainWindow::tick (qint64 time)
{
  QTime t (0, (time / 60000) % 60, (time / 1000) % 60);
  l_time->setText (t.toString ("mm:ss"));
}


void MainWindow::totalTimeChanged (qint64 newTotalTime) 
{
  QTime t (0, (newTotalTime / 60000) % 60, (newTotalTime / 1000) % 60);
  l_total_time->setText (t.toString ("mm:ss"));
}


void MainWindow::sourceChanged (const Phonon::MediaSource &source)
{
  act_play->setIcon (QIcon (":/icons/play.svg"));
    
  QModelIndex index = fman->index_from_name (QFileInfo(mediaObject->currentSource().fileName()).fileName());

  fman->tree_view->selectionModel()->setCurrentIndex (index, QItemSelectionModel::Select | QItemSelectionModel::Current | QItemSelectionModel::Rows);
  fman->tree_view->scrollTo (index);
}


void MainWindow::metaDataChanged()
{
  if (mediaObject->currentSource().type() == Phonon::MediaSource::Invalid)
      return;

  QMap <QString, QString> metaData = mediaObject->metaData();

  QString title = metaData.value ("TITLE");
  if (title.isEmpty())
      title = mediaObject->currentSource().fileName();

  QString output = tags_format;
  QString artist;
  QString album;
  
  if (charset != "UTF-8")
     {
      QTextCodec *codec = QTextCodec::codecForName(charset.toAscii());
      artist = codec->toUnicode (metaData.value ("ARTIST").toAscii());
      album = codec->toUnicode (metaData.value ("ALBUM").toAscii());
      if (! metaData.value ("TITLE").isEmpty())
          title = codec->toUnicode (metaData.value ("TITLE").toAscii());
     }
  else
      {
       artist = metaData.value ("ARTIST");
       album = metaData.value ("ALBUM");
      }
  
  output.replace ("%ARTIST", artist, Qt::CaseInsensitive);
  output.replace ("%TITLE", title, Qt::CaseInsensitive);
  output.replace ("%ALBUM", album, Qt::CaseInsensitive);

  log->log (output);
  setWindowTitle (tr ("/neformal/ >> ") + title);
}
  

void MainWindow::aboutToFinish()
{
  enq_next();
}



void MainWindow::create_manual()
{
  QWidget *wd_man = new QWidget (this);

  QVBoxLayout *lv_t = new QVBoxLayout;

  QString loc = QLocale::system().name().left (2);

  QString filename (":/manuals/");
  filename.append (loc).append (".html");
  
  if (! file_exists (filename))
      filename = ":/manuals/en.html";


  QHBoxLayout *lh_controls = new QHBoxLayout();

  QPushButton *bt_back = new QPushButton ("<");
  QPushButton *bt_forw = new QPushButton (">");

  lh_controls->addWidget (bt_back);
  lh_controls->addWidget (bt_forw);

  man = new QTextBrowser;
  man->setOpenExternalLinks (TRUE);
  man->setSource (QUrl ("qrc" + filename));

  connect (bt_back, SIGNAL(clicked()), man, SLOT(backward()));
  connect (bt_forw, SIGNAL(clicked()), man, SLOT(forward()));

  lv_t->addLayout (lh_controls);
  lv_t->addWidget (man);

  wd_man->setLayout (lv_t);

  main_tab_widget->addTab (wd_man, tr ("learn"));
}



void MainWindow::create_actions()
{
  act_test = new QAction (QIcon (":/icons/play.svg"), tr ("Test"), this);
  connect (act_test, SIGNAL(triggered()), this, SLOT(test()));
  
  
  act_play = new QAction (QIcon (":/icons/play.svg"), tr ("Play"), this);
  connect (act_play, SIGNAL(triggered()), this, SLOT(play()));

  act_stop = new QAction (QIcon (":/icons/stop.svg"), tr ("Stop"), this);
  connect (act_stop, SIGNAL(triggered()), this, SLOT(stop()));

  act_prev = new QAction (QIcon (":/icons/prev.svg"), tr ("Prev"), this);
  connect (act_prev, SIGNAL(triggered()), this, SLOT(hard_prev()));
  
  act_next = new QAction (QIcon (":/icons/next.svg"), tr ("Next"), this);
  connect (act_next, SIGNAL(triggered()), this, SLOT(hard_next()));
  
  act_exit = new QAction (tr ("Exit"), this);
  act_exit->setShortcut (QKeySequence ("Ctrl+Q"));
  act_exit->setStatusTip (tr ("Exit the application"));
  connect (act_exit, SIGNAL(triggered()), this, SLOT(close()));
 
  act_about_qt = new QAction (tr ("About Qt"), this);
  connect (act_about_qt, SIGNAL(triggered()), qApp, SLOT(aboutQt()));
}


QAction* MainWindow::add_to_menu (QMenu *menu,
                                  const QString &caption,
                                  const char *method,
                                  const QString &shortkt,
                                  const QString &iconpath
                                 )
{
  QAction *act = new QAction (caption, this);

  if (! shortkt.isEmpty())
     act->setShortcut (shortkt);

  if (! iconpath.isEmpty())
     act->setIcon (QIcon (iconpath));

  connect (act, SIGNAL(triggered()), this, method);
  menu->addAction (act);
  return act;
}


void MainWindow::create_menus()
{
  fileMenu = menuBar()->addMenu (tr ("File"));
  fileMenu->setTearOffEnabled (TRUE);

//  fileMenu->addAction (act_test);

  add_to_menu (fileMenu, tr ("Prev"), SLOT(hard_prev()),"Ctrl+,");
  add_to_menu (fileMenu, tr ("Stop"), SLOT(stop()), "Ctrl+S");
  add_to_menu (fileMenu, tr ("Play"), SLOT(play()), "Ctrl+P");
  add_to_menu (fileMenu, tr ("Next"), SLOT(hard_next()), "Ctrl+.");
 
  fileMenu->addSeparator();

  add_to_menu (fileMenu, tr ("Refresh  directory"), SLOT(fman_refresh_dir()), "Ctrl+R");
 
  fileMenu->addSeparator();
 
  fileMenu->addAction (act_exit);

  
  viewMenu = menuBar()->addMenu (tr ("View"));
  viewMenu->setTearOffEnabled (true);

  view_themes_menu = viewMenu->addMenu (tr ("Themes"));
  add_to_menu (viewMenu, tr ("Charset"), SLOT(select_enc()));
    

  funcsMenu = menuBar()->addMenu (tr ("Functions"));
  funcsMenu->setTearOffEnabled (true);

  add_to_menu (funcsMenu, tr("File information"), SLOT(fn_get_file_info()));

  funcs_fileops = funcsMenu->addMenu (tr ("File operations"));
  funcs_fileops->setTearOffEnabled (true);
  add_to_menu (funcs_fileops, tr ("Delete"), SLOT(fman_delete()));
  add_to_menu (funcs_fileops, tr ("Rename"), SLOT(fman_rename()));
  //add copy to
  //add move to 
  
  
  helpMenu = menuBar()->addMenu (tr ("Help"));
  helpMenu->setTearOffEnabled (true);

  add_to_menu (helpMenu, "ChangeLog", SLOT(show_changelog()));
  add_to_menu (helpMenu, tr ("License"), SLOT(show_gpl()));
  add_to_menu (helpMenu, tr ("NEWS"), SLOT(show_news()));

  helpMenu->addSeparator();

  helpMenu->addAction (act_about_qt);
  add_to_menu (helpMenu, tr ("About Neformal"), SLOT(about()));
}


void MainWindow::update_places_bookmarks()
{
  lv_places->clear();
  
  if (! file_exists (fname_places_bookmarks))
     return;

  sl_places_bmx = qstring_load (fname_places_bookmarks).split ("\n");
  if (sl_places_bmx.size() != 0)
     lv_places->addItems (sl_places_bmx);
}


void MainWindow::fman_rename()
{
 QString fname = fman->get_sel_fname();
  if (fname.isNull() || fname.isEmpty())
     return;
  
  QFileInfo fi (fname);
  if (! fi.exists() && ! fi.isWritable())
     return;
  
  bool ok;
  QString newname = QInputDialog::getText (this, tr ("Enter the name"),
                                                 tr ("Name:"), QLineEdit::Normal,
                                                 tr ("new"), &ok);
  if (! ok || newname.isEmpty())
     return;
  
  stop();
  QString newfpath (fi.path());
  newfpath.append ("/").append (newname);
  QFile::rename (fname, newfpath);
  fman->refresh();
  QModelIndex index = fman->index_from_name (newname);
  fman->tree_view->selectionModel()->setCurrentIndex (index, QItemSelectionModel::Select);
  fman->tree_view->scrollTo (index, QAbstractItemView::PositionAtCenter);

}


void MainWindow::fman_delete()
{
  QString fname = fman->get_sel_fname();
  if (fname.isNull() || fname.isEmpty())
     return;

  int i = fman->get_sel_index();
  
  QFileInfo fi (fname);
  if (! fi.exists() && ! fi.isWritable())
     return;
 
  if (QMessageBox::warning (this, "Neformal",
                            tr ("Are you sure to delete\n"
                            "%1?").arg (fname),
                            QMessageBox::Yes | QMessageBox::Default,
                            QMessageBox::No | QMessageBox::Escape) == QMessageBox::No)
      return;
 
  stop();
  QFile::remove (fname);
  fman->refresh();
  if (i < fman->list.count())
     {
      QModelIndex index = fman->index_from_idx (i);
      fman->tree_view->selectionModel()->setCurrentIndex (index, QItemSelectionModel::Select);
      fman->tree_view->scrollTo (index, QAbstractItemView::PositionAtCenter);
     }
}


void MainWindow::fman_add_bmk()
{
  sl_places_bmx.prepend (ed_fman_path->text());
  qstring_save (fname_places_bookmarks, sl_places_bmx.join ("\n"));
  update_places_bookmarks();
}


void MainWindow::fman_del_bmk()
{
  int i = lv_places->currentRow();
  if (i == -1)
     return; 

  QString s = lv_places->item(i)->text();
  if (s.isNull() || s.isEmpty())
     return;

  i = sl_places_bmx.indexOf (s);
  sl_places_bmx.removeAt (i);
  qstring_save (fname_places_bookmarks, sl_places_bmx.join ("\n"));
  update_places_bookmarks();
}


void MainWindow::fman_places_itemActivated (QListWidgetItem *item)
{
  fman->nav (item->text());
}


void MainWindow::fman_dir_changed (const QString &full_path)
{
  ui_update = true;
  ed_fman_path->setText (full_path);

#ifdef Q_WS_WIN

  cb_fman_drives->setCurrentIndex (cb_fman_drives->findText (full_path.left(3).toUpper()));

#endif

  ui_update = false;
}


void MainWindow::enq_next()
{
  int index = fman->get_sel_index();
  if (index == -1)
      return;
  
  for (int i = index; i < fman->list.size(); i++)
      {
       QFileInfo fi = fman->list[i];
       if (is_file_supported_by_mime (fi.absoluteFilePath())) 
          { 
           mediaObject->enqueue (fi.absoluteFilePath());
           break;
          }
      }
}


void MainWindow::hard_next()
{
  int index = fman->get_sel_index();
  if (index == -1)
      return;
  
  for (int i = index; i < fman->list.size(); i++)
      {
       QFileInfo fi = fman->list[i];
       if (is_file_supported_by_mime (fi.absoluteFilePath())) 
          { 
           mediaObject->stop();
           mediaObject->queue().clear();
           mediaObject->setCurrentSource (fi.absoluteFilePath());
           mediaObject->play();
  
           break;
          }
      }
}


void MainWindow::hard_prev()
{
  int index = fman->get_sel_index();
  if (index == -1)
      return;
  
  if (! fman->dir.isRoot())
      index -= 2;
  
  for (int i = index; i > -1; i--)
      {
       QFileInfo fi = fman->list[i];
       
       if (is_file_supported_by_mime (fi.absoluteFilePath())) 
          {
           mediaObject->stop();
           mediaObject->queue().clear();
           mediaObject->setCurrentSource (fi.absoluteFilePath());
           mediaObject->play();
           break;
          }
      }
}


void MainWindow::play()
{
  if (mediaObject->state() == Phonon::PausedState) 
     {
      mediaObject->play();
      act_play->setIcon (QIcon (":/icons/play.svg"));
     }
  else
  if (mediaObject->state() == Phonon::PlayingState) 
    {
     mediaObject->pause();
     act_play->setIcon (QIcon (":/icons/pause.svg"));
    }
  else
  if (mediaObject->state() == Phonon::StoppedState || Phonon::ErrorState) 
     {
      int index = fman->get_sel_index();
      if (index == -1)
         return;
     
     if (! fman->dir.isRoot())
        index--; 
     
//      qDebug() << "play " << fman->list[index].absoluteFilePath();
           
      mediaObject->queue().clear();
      mediaObject->setCurrentSource (fman->list[index].absoluteFilePath());
      mediaObject->play();
     }
} 


void MainWindow::stop()
{
  mediaObject->stop();
  mediaObject->queue().clear();
} 


void MainWindow::update_themes_menu()
{
  view_themes_menu->clear();
  create_menu_from_list (this, view_themes_menu,
                         QStyleFactory::keys(),
                         SLOT (use_theme_triggered()));
}


void MainWindow::use_theme_triggered()
{
  QAction *a = qobject_cast<QAction *>(sender());
  
  QApplication::setStyle (QStyleFactory::create (a->text()));
  settings->setValue ("ui_theme", a->text());
}


void MainWindow::setup_mime_table()
{
  mime_types = hash_load_keyval (":/res/mime.txt");
  mime_types_supported = stringlist_to_hash (Phonon::BackendCapabilities::availableMimeTypes());
}


bool MainWindow::is_file_supported_by_mime (const QString &fname)
{
  QString ext = file_get_ext (fname);
  if (ext.isEmpty())
     return false;
  
  QList <QString> mts = mime_types.values (ext);
  if (mime_types.count() == 0)
     return false;
   
  foreach (QString mt, mts)
          {
           if (mime_types_supported.contains (mt))
               return true;
          }
  
  return false;
}


void MainWindow::show_text_file (const QString &fname)
{
  text_file_browser->setPlainText (qstring_load (fname));
  text_file_browser->resize (width(), height());
  text_file_browser->show();
}


void MainWindow::show_text_file_enc (const QString &fname)
{
  text_file_browser->setPlainText (qstring_load (fname, charset));
  text_file_browser->resize (width(), height());
  text_file_browser->show();
}


void MainWindow::show_changelog()
{
  show_text_file (":/ChangeLog");
}


void MainWindow::show_gpl()
{
  show_text_file (":/COPYING");
}


void MainWindow::show_news()
{
 
  QString fname = ":/NEWS";
  if (QLocale::system().name().left(2) == "ru")
     fname = ":/NEWS-RU";

  show_text_file  (fname);
}



CReloadWindow::CReloadWindow (const QString &def)
{
  setAttribute (Qt::WA_DeleteOnClose);
  QVBoxLayout *lt = new QVBoxLayout;

  QLabel *l = new QLabel (tr ("Charset"));

  list = new QListWidget (this);
   
  lt->addWidget (l);
  lt->addWidget (list);

  QStringList sl;
  foreach (QByteArray codec, QTextCodec::availableCodecs())
          sl.prepend (codec);
  
  sl.sort();
  
  list->addItems (sl);
  
  
  list->setCurrentRow (sl.indexOf (def));
  
  setLayout (lt);
  setWindowTitle (tr ("Select the charset"));
}


void CReloadWindow::closeEvent (QCloseEvent *event)
{
  event->accept();
}


void MainWindow::enc_itemDoubleClicked (QListWidgetItem *item)
{
  charset = item->text();
  qDebug () << charset;
  enc_window->close();
}


void MainWindow::select_enc()
{
  enc_window = new CReloadWindow (charset);
     
  connect (enc_window->list, SIGNAL(itemDoubleClicked (QListWidgetItem *)), 
           this, SLOT(enc_itemDoubleClicked (QListWidgetItem *)));

  enc_window->show();
}



void CAboutWindow::closeEvent (QCloseEvent *event)
{
  event->accept();
}


CAboutWindow::CAboutWindow()
{
  setAttribute (Qt::WA_DeleteOnClose);

  QStringList sl_t = qstring_load (":/AUTHORS").split ("##");

  logo = new QLabel;
  logo->setPixmap (QPixmap (":/res/nefor_toplogo.png"));
 
  
  QTabWidget *tw = new QTabWidget (this);

  QPlainTextEdit *page_code = new QPlainTextEdit();
  QPlainTextEdit *page_thanks = new QPlainTextEdit();
  QPlainTextEdit *page_translators = new QPlainTextEdit();
  QPlainTextEdit *page_maintainers = new QPlainTextEdit();

  if (sl_t.size() == 4)
     {
      page_code->setPlainText (sl_t[0].trimmed());
      page_thanks->setPlainText (sl_t[3].trimmed());
      page_translators->setPlainText (sl_t[1].trimmed());
      page_maintainers->setPlainText (sl_t[2].trimmed());
     }

  tw->addTab (page_code, tr ("Code"));
  tw->addTab (page_thanks, tr ("Acknowledgements"));
  tw->addTab (page_translators, tr ("Translations"));
  tw->addTab (page_maintainers, tr ("Packages"));

  QVBoxLayout *layout = new QVBoxLayout();

  layout->addWidget(logo);
  layout->addWidget(tw);

  setLayout (layout);
  setWindowTitle (tr ("About"));
}


void MainWindow::about()
{
  CAboutWindow *a = new CAboutWindow();
  a->move (x() + 20, y() + 20);
  a->show();
}


void MainWindow::fn_get_file_info()
{
  QString fname = fman->get_sel_fname();
  if (fname.isEmpty())
     return;

  QFileInfo fi (fname);
  if (! fi.exists())
     return;

  QStringList l;

  l.append (tr ("file name: %1").arg (fi.absoluteFilePath()));
  
  if (file_get_ext (fname) == "wav")
     {
      CWavReader wr;
      wr.get_info (fname);
      if (wr.wav_chunk_fmt.bits_per_sample == 16)
         l.append (tr ("RMS for all channels: %1 dB").arg (wr.rms));

      l.append (tr ("bits per sample: %1").arg (wr.wav_chunk_fmt.bits_per_sample));
      l.append (tr ("number of channels: %1").arg (wr.wav_chunk_fmt.num_channels));
      l.append (tr ("sample rate: %1").arg (wr.wav_chunk_fmt.sample_rate));
     }
  
  l.append (tr ("size: %1 kbytes").arg (QString::number (fi.size() / 1024)));
  l.append (tr ("created: %1").arg (fi.created().toString ("yyyy-MM-dd@hh:mm:ss")));
  l.append (tr ("modified: %1").arg (fi.lastModified().toString ("yyyy-MM-dd@hh:mm:ss")));
  
  log->log (l.join ("<br>"));
}


CLogMemo::CLogMemo (QPlainTextEdit *m)
{
  memo = m;
  setObjectName ("logmemo");
  memo->setFocusPolicy (Qt::ClickFocus);
}


void CLogMemo::log (const QString &text)
{
  if (memo->toPlainText().count() > 999)
     memo->clear();
 
  QTextCursor cr = memo->textCursor();
  cr.movePosition (QTextCursor::Start);
  cr.movePosition (QTextCursor::Down, QTextCursor::MoveAnchor, 0);
  memo->setTextCursor (cr);

  QTime t = QTime::currentTime();

  memo->textCursor().insertHtml ("[" + t.toString("hh:mm:ss") + "] " + text + "<br>");

  cr = memo->textCursor();
  cr.movePosition (QTextCursor::Start);
  cr.movePosition (QTextCursor::Down, QTextCursor::MoveAnchor, 0);
  memo->setTextCursor (cr);
}


void MainWindow::test()
{
  qstring_list_print (mime_types_supported.keys());
}

